@@ -16,6 +16,7 @@ from .. util.iter import index_sequence
 from .. util.iter import depth_one
 from .. util.iter import ordered_unique
 from .. util.iter import is_vengeance_class
+from .. util.iter import is_flux_row_class
 
 from .. util.text import repr_
 from .. util.text import p_json_dumps
@@ -29,20 +30,16 @@ from . flux_row_cls import flux_row_cls
 
 class flux_cls:
 
-    def __init__(self, matrix):
-        if is_vengeance_class(matrix):
-            matrix = matrix.rows()
+    def __init__(self, matrix=None):
+        self._num_cols = None
+        self.is_empty  = None
 
         self.headers = OrderedDict()
-
-        self._num_cols = None
-        self._matrix   = None
-
-        self.__to_flux_rows(matrix)
+        self.matrix  = self.__to_flux_rows(matrix)
 
     @property
     def header_values(self):
-        return self._matrix[0].values
+        return self.matrix[0].values
 
     @property
     def num_cols(self):
@@ -50,27 +47,27 @@ class flux_cls:
         if isinstance(self._num_cols, int):
             return self._num_cols
 
-        self._num_cols = max(len(row) for row in self._matrix)
+        self._num_cols = max(len(row) for row in self.matrix)
 
         return self._num_cols
 
     @property
     def is_jagged(self):
-        num_cols = len(self._matrix[0])
-        for row in self._matrix[1:]:
+        num_cols = len(self.matrix[0])
+        for row in self.matrix[1:]:
             if len(row) != num_cols:
                 return True
 
         return False
 
     @property
-    def is_empty(self):
+    def has_data(self):
         return self.num_rows == 0
 
     @property
     def num_rows(self):
         """ header row not included in count """
-        return len(self._matrix) - 1
+        return len(self.matrix) - 1
 
     def execute_commands(self, commands, profile_performance=False):
         if profile_performance:
@@ -132,7 +129,7 @@ class flux_cls:
             self.headers[header] = i
 
     def matrix_by_headers(self, *headers):
-        columns = transpose([row.values for row in self._matrix[1:]])
+        columns = transpose([row.values for row in self.matrix[1:]])
 
         m = []
         for header in modify_iteration_depth(headers, 1):
@@ -142,13 +139,15 @@ class flux_cls:
                 column = [None] * self.num_rows
             else:
                 i = self.headers[header]
-                column = columns[i]
+                column = columns[i].copy()
 
             column.insert(0, header)
             m.append(column)
 
         m = transpose(m)
-        self.__to_flux_rows(m)
+
+        self.headers.clear()
+        self.matrix = self.__to_flux_rows(m)
 
     def __process_header(self, header):
         """
@@ -159,7 +158,7 @@ class flux_cls:
         # renamed header
         if isinstance(header, dict):
             h_old, h_new = tuple(header.items())[0]
-            self.headers[h_new] = self.headers.pop(h_old)
+            self.headers[h_new] = self.headers[h_old]
 
             return h_new
 
@@ -240,7 +239,7 @@ class flux_cls:
 
     def append_columns(self, *names):
         names = modify_iteration_depth(names, depth=1)
-        self._matrix[0].values.extend(names)
+        self.matrix[0].values.extend(names)
 
         for header in names:
             self.headers[header] = len(self.headers)
@@ -252,55 +251,33 @@ class flux_cls:
         indeces = [self.headers[h] for h in names]
 
         for c in sorted(indeces, reverse=True):
-            for row in self._matrix:
+            for row in self.matrix:
                 del row.values[c]
 
         self.reapply_header_names(self.header_values)
 
-    def insert_rows(self, i, rows):
-        if is_vengeance_class(rows):
-            if i == 0:
-                rows = list(rows.rows())
-            else:
-                rows = list(rows.rows())[1:]
-
-        rows = generator_to_list(rows)
-        assert_iteration_depth(rows, 2)
+    def insert_rows(self, i, m):
+        if self.is_empty:
+            self.matrix = self.__to_flux_rows(m)
+            return
 
         if i == 0:
-            self.reapply_header_names(rows[0])
-            del self._matrix[0]
+            self.headers.clear()
+            del self.matrix[0]
+        elif is_vengeance_class(m):
+            m = list(m.rows())[1:]
 
-        self._num_cols = None
+        self.matrix[i:i] = self.__to_flux_rows(m)
 
-        m = []
-        for row in rows:
-            if isinstance(row, flux_row_cls):
-                row = row.values
+    def append_rows(self, m):
+        if self.is_empty:
+            self.matrix = self.__to_flux_rows(m)
+            return
 
-            row = flux_row_cls(self.headers, row)
-            m.append(row)
+        if is_vengeance_class(m):
+            m = list(m.rows())[1:]
 
-        self._matrix[i:i] = m
-
-    def append_rows(self, rows):
-        if is_vengeance_class(rows):
-            rows = list(rows.rows())[1:]
-
-        rows = generator_to_list(rows)
-        assert_iteration_depth(rows, 2)
-
-        if len(self._matrix) == 0:
-            self.headers = index_sequence(str(v) for v in rows[0])
-
-        self._num_cols = None
-
-        for row in rows:
-            if isinstance(row, flux_row_cls):
-                row = row.values
-
-            row = flux_row_cls(self.headers, row)
-            self._matrix.append(row)
+        self.matrix.extend(self.__to_flux_rows(m))
 
     def fill_jagged_columns(self):
         num_cols = self.num_cols
@@ -318,7 +295,7 @@ class flux_cls:
             flux.sort('col_a', 'col_b', 'col_c',
                       reverse=[True, True, True])
         """
-        self._matrix[1:] = self.__sort_rows(f, kwargs)
+        self.matrix[1:] = self.__sort_rows(f, kwargs)
 
     def sorted(self, *f, **kwargs):
         """ returns new flux
@@ -333,7 +310,7 @@ class flux_cls:
 
     def filter(self, f):
         """ in-place """
-        self._matrix[1:] = [row for row in self if f(row)]
+        self.matrix[1:] = [row for row in self if f(row)]
 
     def filtered(self, f):
         """ return new flux """
@@ -387,10 +364,10 @@ class flux_cls:
         return ordered_unique(f(row) for row in self)
 
     def reset_matrix(self, m):
-        self._matrix   = None
         self._num_cols = None
 
-        self.__to_flux_rows(m)
+        self.headers.clear()
+        self.matrix = self.__to_flux_rows(m)
 
     def index_row(self, *f):
         """ dictionary of {f(row): row}
@@ -423,25 +400,39 @@ class flux_cls:
         """ speed up attribute access by binding values directly to flux_row_cls.__dict__
         (only use this if you know the side-effects)
         """
-        [row.bind() for row in self._matrix]
+        [row.bind() for row in self.matrix]
 
     def unbind(self):
         """ return dynamic flux_row_cls.values from flux_row_cls.__dict__ """
         instance_names = tuple(self.headers.keys())
-        [row.unbind(instance_names) for row in self._matrix]
+        [row.unbind(instance_names) for row in self.matrix]
+
+    def to_csv(self, path, encoding=None):
+        path = apply_file_extension(path, '.csv')
+        m = list(self.rows())
+
+        write_file(path, m, encoding=encoding)
+
+    @classmethod
+    def from_csv(cls, path, encoding=None):
+        path = apply_file_extension(path, '.csv')
+        m = read_file(path, encoding=encoding)
+
+        return cls(m)
+
+    def to_json(self, path, encoding=None):
+        path = apply_file_extension(path, '.json')
 
-    def to_json(self, path=None):
         j = [row.dict() for row in self]
         j = p_json_dumps(j)
 
-        if path is None:
-            return j
-
-        write_file(path, j)
+        write_file(path, j, encoding=encoding)
 
     @classmethod
-    def from_json(cls, path):
-        rows = read_file(path)
+    def from_json(cls, path, encoding=None):
+        path = apply_file_extension(path, '.json')
+
+        rows = read_file(path, encoding=encoding)
         if rows:
             m = [list(d.values()) for d in rows]
             m.insert(0, list(rows[0].keys()))
@@ -450,15 +441,6 @@ class flux_cls:
 
         return cls(m)
 
-    def to_csv(self, path):
-        m = list(self.rows())
-        write_file(path, m)
-
-    @classmethod
-    def from_csv(cls, path):
-        m = read_file(path)
-        return cls(m)
-
     def serialize(self, path):
         path = apply_file_extension(path, '.flux')
         write_file(path, self)
@@ -472,37 +454,56 @@ class flux_cls:
         r_1 = self.__matrix_index(r_1)
         r_2 = self.__matrix_index(r_2)
 
-        return (row.values for row in islice(self._matrix, r_1, r_2 + 1))
+        return (row.values for row in islice(self.matrix, r_1, r_2 + 1))
 
     def flux_rows(self, r_1='*h', r_2='*l'):
         r_1 = self.__matrix_index(r_1)
         r_2 = self.__matrix_index(r_2)
 
-        return iter(islice(self._matrix, r_1, r_2 + 1))
+        return iter(islice(self.matrix, r_1, r_2 + 1))
 
     def __to_flux_rows(self, m):
         m = generator_to_list(m)
+
+        if m is None or not any(m):
+            self.is_empty = True
+            return [flux_row_cls({}, [])]
+
         assert_iteration_depth(m, 2)
 
-        reserved = set(m[0]) & flux_row_cls.class_names
-        if reserved:
-            raise NameError("reserved name(s) {} found in header row {}".format(list(reserved), m[0]))
+        if is_flux_row_class(m[0]):
+            m = [row.values for row in m]
 
-        self.headers = index_sequence(str(v) for v in m[0])
-        self._matrix = []
+        self.__assert_no_reserved_headers(m[0])
+        if not self.headers:
+            self.headers = index_sequence(str(v) for v in m[0])
+
+        num_cols = self._num_cols or 0
+        flux_m   = []
 
         for row in m:
             if not isinstance(row, list):
                 raise ValueError('row must be list')
 
-            row_o = flux_row_cls(self.headers, row)
-            self._matrix.append(row_o)
+            num_cols = max(num_cols, len(row))
+            flux_m.append(flux_row_cls(self.headers, row))
+
+        self._num_cols = num_cols
+        self.is_empty  = (len(flux_m) == 0)
+
+        return flux_m
+
+    @staticmethod
+    def __assert_no_reserved_headers(headers):
+        reserved = set(headers) & flux_row_cls.class_names
+        if reserved:
+            raise NameError("reserved name(s) {} found in header row {}".format(list(reserved), headers))
 
     def __to_primitive_rows(self):
-        if not isinstance(self._matrix[0], flux_row_cls):
+        if not isinstance(self.matrix[0], flux_row_cls):
             return
 
-        self._matrix = [row.values for row in self._matrix]
+        self.matrix = [row.values for row in self.matrix]
 
     def __row_values_accessor(self, f):
         """ convert f into a function that can be called on each row in self._matrix """
@@ -522,7 +523,7 @@ class flux_cls:
             return f
 
         if isinstance(f, (list, tuple)):
-            columns = tuple(self.headers[n] for n in f)
+            columns = tuple(self.headers.get(n, n) for n in f)
             f = row_values
         elif isinstance(f, str):
             i = self.headers[f]
@@ -534,13 +535,25 @@ class flux_cls:
         return f
 
     def __validate_row_accessor(self, f):
+        num_cols = len(self.headers) - 1
+
         if isinstance(f, (str, list, tuple)):
-            invalid = [h for h in depth_one(f) if h not in self.headers]
+            f_ = depth_one(f)
+
+            invalid = [h for h in f_
+                         if isinstance(h, str)
+                         if h not in self.headers]
             if invalid:
                 raise KeyError("invalid column reference: {}".format(repr_(invalid)))
 
+            invalid = [i for i in f_
+                         if isinstance(i, int)
+                         if i > num_cols]
+            if invalid:
+                raise KeyError("integer(s) out of bounds: {}".format(repr_(invalid)))
+
         elif isinstance(f, int):
-            if f >= len(self.headers):
+            if f > num_cols:
                 raise KeyError("integer out of bounds: {}".format(f))
 
         else:
@@ -568,15 +581,15 @@ class flux_cls:
             return deepcopy(self)
 
         # self._num_cols?
-        m = [row.values for row in self._matrix]
+        m = [row.values for row in self.matrix]
         return self.__class__(m.copy())
 
     def __len__(self):
-        return len(self._matrix)
+        return len(self.matrix)
 
     def __getitem__(self, ref):
         if isinstance(ref, int):
-            return self._matrix[ref]
+            return self.matrix[ref]
 
         if isinstance(ref, str):
             if ref not in self.headers:
@@ -589,19 +602,19 @@ class flux_cls:
             i_1, i_2, step = ref.start, ref.stop, ref.step
             if i_1 is not None:
                 if i_1 < 0:
-                    i_1 = len(self._matrix) + i_1
+                    i_1 = len(self.matrix) + i_1
 
             if i_2 is not None:
                 if i_2 < 0:
-                    i_2 = len(self._matrix) + i_2
+                    i_2 = len(self.matrix) + i_2
 
-            return list(row for row in islice(self._matrix, i_1, i_2, step))
+            return list(row for row in islice(self.matrix, i_1, i_2, step))
 
         raise KeyError("reference syntax should be integers, eg 'flux[1:3]'")
 
     def __setitem__(self, ref, column):
         if ref not in self.headers:
-            raise KeyError("column '{}' not in headers".format(ref))
+            self.append_columns(ref)
 
         column = modify_iteration_depth(column, 1)
         nd = iteration_depth(column)
@@ -619,7 +632,7 @@ class flux_cls:
             row.values[i] = v
 
     def __iter__(self):
-        return iter(islice(self._matrix, 1, None))
+        return iter(islice(self.matrix, 1, None))
 
     def __repr__(self):
         headers = repr_(self.headers.keys(), wrap='[]')
