@@ -1,6 +1,8 @@
 
 import copy
 
+from math import ceil
+
 from types import GeneratorType
 
 from collections import OrderedDict
@@ -21,14 +23,14 @@ def modify_iteration_depth(v, depth=0):
     nd = iteration_depth(v)
 
     if nd > depth:
-        # remove nesting
-        for _ in range(nd - depth + 1):
-            if len(v) == 1 and is_subscriptable(v):
+        # remove unneccesary nesting
+        for _ in range(nd - depth):
+            if is_subscriptable(v) and len(v) == 1:
                 v = list(v)
                 v = v[0]
 
     elif nd < depth:
-        # additional nesting
+        # add additional nesting
         for _ in range(depth - nd):
             v = [v]
 
@@ -36,9 +38,19 @@ def modify_iteration_depth(v, depth=0):
 
 
 def assert_iteration_depth(v, depth):
+    """ make sure values have certain number of iterable layers """
     nd = iteration_depth(v)
+
     if nd != depth:
-        raise IndexError('invalid iteration depth, must be of depth {}'.format(depth))
+        msg = ('invalid iteration depth {}, value must have {} layers of iteration'
+               .format(nd, depth))
+
+        if depth == 1:
+            msg += ', (ie, a list)'
+        elif depth == 2:
+            msg += ', (ie, list of lists)'
+
+        raise IndexError(msg)
 
 
 def iteration_depth(v):
@@ -122,6 +134,24 @@ def generator_to_list(v, recurse=False):
     return v
 
 
+def sequence_strides(sequence, num_strides):
+    if isinstance(sequence, int):
+        sequence = [None] * sequence
+    else:
+        sequence = generator_to_list(sequence)
+
+    num_items  = len(sequence)
+    stride_len = int(ceil(num_items / num_strides))
+    stride_len = max(stride_len, 1)
+
+    strides = []
+    for i_1 in range(0, num_items, stride_len):
+        i_2 = i_1 + stride_len
+        strides.append(sequence[i_1:i_2])
+
+    return strides
+
+
 def is_vengeance_class(o):
     bases = {b.__name__ for b in o.__class__.mro()}
     if 'flux_cls' in bases or 'excel_levity_cls' in bases:
@@ -130,6 +160,14 @@ def is_vengeance_class(o):
     return False
 
 
+def is_flux_row_class(o):
+    bases = {b.__name__ for b in o.__class__.mro()}
+    if 'flux_row_cls' in bases or 'lev_row_cls' in bases:
+        return True
+
+    return False
+
+
 def transpose(m):
     m = generator_to_list(m)
 
